#!/usr/bin/perl

use PDF::API2;

use constant mm => 25.4/72;
use constant cm => 2.54/72;
use constant in =>    1/72;
use constant pt =>    1;

$source = PDF::API2->open("document.pdf");
$dest = PDF::API2->new;

$left = 1; 
$right = ($source->pages % 4 == 0) ? $source->pages : $source->pages + (4 - $source->pages % 4); 
$reversed = 1;
while ($left < $right) {
  $page = $dest->page;
  $page->mediabox(297/mm, 210/mm);
  $page->rotate(90);

  draw_page($source, $left, $dest, -1, $reversed);
  draw_page($source, $right, $dest, -1, !$reversed) if ($right <= $source->pages);
  
  $left++; $right--; $reversed = !$reversed;
}

$dest->saveas("document_book.pdf");

$dest->end;
$source->end;

sub draw_page {
  my ($pdf_in, $in_idx, $pdf_out, $out_idx, $position) = @_;

  my $xo = $pdf_out->importPageIntoForm($pdf_in, $in_idx);
  my $pg_out = $pdf_out->openpage($out_idx);  

  if (my $cropbox = $pdf_in->openpage($in_idx)->find_prop("CropBox")) {
    $xo->bbox(map {$_->val} $cropbox->elementsof);
  }

  my @ps = map {$_->val} $xo->{BBox}->elementsof;
  my $bbox_width = $ps[2] - $ps[0];
  my $bbox_height = $ps[3] - $ps[1];

  my (undef, undef, $page_width, $page_height) = $pg_out->get_mediabox;

  my $scale_x = $page_width/(2*$bbox_width);
  my $scale_y = $page_height/$bbox_height;
  my ($scale, $x, $y);

  if ($scale_x <= $scale_y) {
    $scale = $scale_x;
    $x = 0;
    $y = ($page_height - $scale*$bbox_height)/2;
  } 
  else {
    $scale = $scale_y;
    $x = ($page_width/2 - $scale*$bbox_width)/2;
    $y = 0;
  }

  $pg_out->gfx->formimage($xo, $x - $ps[0] + ($position ? $page_width/2 : 0), $y - $ps[1], $scale);
}
